@@ -2,9 +2,9 @@
 #
 #  Net::Server::Proto::SSLEAY - Net::Server Protocol module
 #
-#  $Id: SSLEAY.pm,v 1.6 2010/07/09 15:05:05 rhandom Exp $
+#  $Id: SSLEAY.pm,v 1.22 2012/05/29 22:54:31 rhandom Exp $
 #
-#  Copyright (C) 2010
+#  Copyright (C) 2010-2012
 #
 #    Paul Seamons
 #    paul@seamons.com
@@ -22,114 +22,128 @@
 package Net::Server::Proto::SSLEAY;
 
 use strict;
-use vars qw($VERSION $AUTOLOAD @ISA);
+use warnings;
 use IO::Socket::INET;
 use Fcntl ();
 use Errno ();
 use Socket ();
 
 BEGIN {
-    eval { require Net::SSLeay };
-    $@ && warn "Module Net::SSLeay is required for SSLeay.";
-    # Net::SSLeay gets mad if we call these multiple times - the question is - who will call them multiple times?
+    eval { require Net::SSLeay; 1 }
+        or warn "Module Net::SSLeay is required for SSLeay.";
     for my $sub (qw(load_error_strings SSLeay_add_ssl_algorithms ENGINE_load_builtin_engines ENGINE_register_all_complete randomize)) {
         Net::SSLeay->can($sub)->();
     }
 }
 
-$VERSION = $Net::Server::VERSION; # done until separated
-@ISA = qw(IO::Socket::INET);
+our @ISA = qw(IO::Socket::INET);
+our $AUTOLOAD;
+
+my @ssl_args = qw(
+    SSL_use_cert
+    SSL_verify_mode
+    SSL_key_file
+    SSL_cert_file
+    SSL_ca_path
+    SSL_ca_file
+    SSL_cipher_list
+    SSL_passwd_cb
+    SSL_max_getline_length
+    SSL_error_callback
+);
+
+sub NS_proto { 'SSLEAY' }
+sub NS_port   { my $sock = shift; ${*$sock}{'NS_port'}   = shift if @_; return ${*$sock}{'NS_port'}   }
+sub NS_host   { my $sock = shift; ${*$sock}{'NS_host'}   = shift if @_; return ${*$sock}{'NS_host'}   }
+sub NS_ipv    { my $sock = shift; ${*$sock}{'NS_ipv'}    = shift if @_; return ${*$sock}{'NS_ipv'}    }
+sub NS_listen { my $sock = shift; ${*$sock}{'NS_listen'} = shift if @_; return ${*$sock}{'NS_listen'} }
 
 sub object {
-    my $type  = shift;
-    my $class = ref($type) || $type || __PACKAGE__;
+    my ($class, $info, $server) = @_;
 
-    my ($default_host,$port,$server) = @_;
-    my $prop = $server->{'server'};
-    my $host;
+    my $ssl = $server->{'server'}->{'ssl_args'} ||= do {
+        my %temp = map {$_ => undef} @ssl_args;
+        $server->configure({map {$_ => \$temp{$_}} @ssl_args});
+        \%temp;
+    };
 
-    if ($port =~ m/^([\w\.\-\*\/]+):(\w+)$/) { # allow for things like "domain.com:80"
-        ($host, $port) = ($1, $2);
+    # we cannot do this at compile time because we have not yet read the configuration then
+    @ISA = qw(IO::Socket::INET6) if $ISA[0] eq 'IO::Socket::INET' && Net::Server::Proto->requires_ipv6($server);
+
+    my @sock = $class->SUPER::new();
+    foreach my $sock (@sock) {
+        $sock->NS_host($info->{'host'});
+        $sock->NS_port($info->{'port'});
+        $sock->NS_ipv( $info->{'ipv'} );
+        $sock->NS_listen(defined($info->{'listen'}) ? $info->{'listen'}
+                        : defined($server->{'server'}->{'listen'}) ? $server->{'server'}->{'listen'}
+                        : Socket::SOMAXCONN());
+
+        for my $key (@ssl_args) {
+            my $val = defined($info->{$key}) ? $info->{$key}
+                    : defined($ssl->{$key})  ? $ssl->{$key}
+                    : $server->can($key) ? $server->$key($info->{'host'}, $info->{'port'}, 'SSLEAY')
+                    : undef;
+            next if ! defined $val;
+            $sock->$key($val) if defined $val;
+        }
     }
-    elsif ($port =~ /^(\w+)$/) { # allow for things like "80"
-        ($host, $port) = ($default_host, $1);
-    }
-    else {
-        $server->fatal("Undeterminate port \"$port\" under ".__PACKAGE__);
-    }
-
-    # read any additional protocol specific arguments
-    my @ssl_args = qw(
-        SSL_server
-        SSL_use_cert
-        SSL_verify_mode
-        SSL_key_file
-        SSL_cert_file
-        SSL_ca_path
-        SSL_ca_file
-        SSL_cipher_list
-        SSL_passwd_cb
-        SSL_error_callback
-        SSL_max_getline_length
-    );
-    my %args;
-    $args{$_} = \$prop->{$_} for @ssl_args;
-    $server->configure(\%args);
-
-    my $sock = $class->new;
-    $sock->NS_host($host);
-    $sock->NS_port($port);
-    $sock->NS_proto('SSLEAY');
-
-    for my $key (@ssl_args) {
-        my $val = defined($prop->{$key}) ? $prop->{$key} : $server->can($key) ? $server->$key($host, $port, 'SSLEAY') : undef;
-        $sock->$key($val);
-    }
-
-    return $sock;
+    return wantarray ? @sock : $sock[0];
 }
 
 sub log_connect {
-    my $sock = shift;
-    my $server = shift;
-    my $host   = $sock->NS_host;
-    my $port   = $sock->NS_port;
-    my $proto  = $sock->NS_proto;
-    $server->log(2,"Binding to $proto port $port on host $host\n");
+    my ($sock, $server) = @_;
+    $server->log(2, "Binding to ".$sock->NS_proto." port ".$sock->NS_port." on host ".$sock->NS_host." with IPv".$sock->NS_ipv);
 }
 
-###----------------------------------------------------------------###
-
 sub connect { # connect the first time
-    my $sock   = shift;
-    my $server = shift;
-    my $prop   = $server->{'server'};
+    my ($sock, $server) = @_;
+    my $host = $sock->NS_host;
+    my $port = $sock->NS_port;
+    my $ipv  = $sock->NS_ipv;
+    my $lstn = $sock->NS_listen;
 
-    my $host  = $sock->NS_host;
-    my $port  = $sock->NS_port;
+    $sock->SUPER::configure({
+        LocalPort => $port,
+        Proto     => 'tcp',
+        Listen    => $lstn,
+        ReuseAddr => 1,
+        Reuse     => 1,
+        (($host ne '*') ? (LocalAddr => $host) : ()), # * is all
+        ($sock->isa("IO::Socket::INET6") ? (Domain => ($ipv eq '6') ? Socket6::AF_INET6() : ($ipv eq '4') ? Socket::AF_INET() : Socket::AF_UNSPEC()) : ()),
+    }) || $server->fatal("Can't connect to SSLEAY port $port on $host [$!]");
 
-    my %args;
-    $args{'LocalPort'} = $port;
-    $args{'Proto'}     = 'tcp';
-    $args{'LocalAddr'} = $host if $host !~ /\*/; # * is all
-    $args{'Listen'}    = $prop->{'listen'};
-    $args{'Reuse'}     = 1;
-
-    $sock->SUPER::configure(\%args) || $server->fatal("Can't connect to SSL port $port on $host [$!]");
-    $server->fatal("Bad sock [$!]!".caller()) if ! $sock;
-
-    if ($port == 0 && ($port = $sock->sockport)) {
+    if ($port eq '0' and $port = $sock->sockport) {
+        $server->log(2, "  Bound to auto-assigned port $port");
+        ${*$sock}{'NS_orig_port'} = $sock->NS_port;
+        $sock->NS_port($port);
+    } elsif ($port =~ /\D/ and $port = $sock->sockport) {
+        $server->log(2, "  Bound to service port ".$sock->NS_port()."($port)");
+        ${*$sock}{'NS_orig_port'} = $sock->NS_port;
         $sock->NS_port($port);
-        $server->log(2,"Bound to auto-assigned port $port");
     }
 
     $sock->bind_SSL($server);
 }
 
 sub reconnect { # connect on a sig -HUP
-    my ($sock, $fd, $server) = @_;
-    my $resp = $sock->fdopen( $fd, 'w' ) || $server->fatal("Error opening to file descriptor ($fd) [$!]");
+    my ($sock, $fd, $server, $port) = @_;
+    $server->log(3,"Reassociating file descriptor $fd with ".$sock->NS_proto." on [".$sock->NS_host."]:".$sock->NS_port.", using IPv".$sock->NS_ipv);
+    my $resp = $sock->fdopen( $fd, 'w' ) or $server->fatal("Error opening to file descriptor ($fd) [$!]");
+
+    if ($sock->isa("IO::Socket::INET6")) {
+        my $ipv = $sock->NS_ipv;
+        ${*$sock}{'io_socket_domain'} = ($ipv eq '6') ? Socket6::AF_INET6() : ($ipv eq '4') ? Socket::AF_INET() : Socket::AF_UNSPEC();
+    }
+
     $sock->bind_SSL($server);
+
+    if ($port ne $sock->NS_port) {
+        $server->log(2, "  Re-bound to previously assigned port $port");
+        ${*$sock}{'NS_orig_port'} = $sock->NS_port;
+        $sock->NS_port($port);
+    }
+
     return $resp;
 }
 
@@ -144,8 +158,8 @@ sub bind_SSL {
     Net::SSLeay::CTX_set_mode($ctx, 0x11);  $sock->SSLeay_check_fatal("SSLeay bind_SSL CTX_set_mode");
 
     # Load certificate. This will prompt for a password if necessary.
-    my $file_key  = $sock->SSL_key_file  || die "SSLeay missing SSL_key_file.\n";
-    my $file_cert = $sock->SSL_cert_file || die "SSLeay missing SSL_cert_file.\n";
+    my $file_key  = $sock->SSL_key_file  || die "SSLeay missing SSL_key_file on ".$sock->hup_string.".\n";
+    my $file_cert = $sock->SSL_cert_file || die "SSLeay missing SSL_cert_file on ".$sock->hup_string>".\n";
     Net::SSLeay::CTX_use_RSAPrivateKey_file($ctx, $file_key,  Net::SSLeay::FILETYPE_PEM());  $sock->SSLeay_check_fatal("SSLeay bind_SSL CTX_use_RSAPrivateKey_file");
     Net::SSLeay::CTX_use_certificate_file(  $ctx, $file_cert, Net::SSLeay::FILETYPE_PEM());  $sock->SSLeay_check_fatal("SSLeay bind_SSL CTX_use_certificate_file");
     $sock->SSLeay_context($ctx);
@@ -163,15 +177,28 @@ sub close {
 }
 
 sub accept {
-    my $sock = shift;
-    my $client = $sock->SUPER::accept;
+    my ($sock, $class) = (@_);
+    my ($client, $peername);
+    if (wantarray) {
+        ($client, $peername) = $sock->SUPER::accept($class);
+    } else {
+        $client = $sock->SUPER::accept($class);
+    }
     if (defined $client) {
         $client->NS_proto($sock->NS_proto);
+        $client->NS_ipv(  $sock->NS_ipv);
+        $client->NS_host( $sock->NS_host);
+        $client->NS_port( $sock->NS_port);
         $client->SSLeay_context($sock->SSLeay_context);
         $client->SSLeay_is_client(1);
     }
 
-    return $client;
+    return wantarray ? ($client, $peername) : $client;
+}
+
+sub post_accept {
+    my $client = shift;
+    $client->SSLeay;
 }
 
 sub SSLeay {
@@ -191,6 +218,7 @@ sub SSLeay {
         ${*$client}{'SSLeay'} = $ssl;
     }
 
+    return if ! defined wantarray;
     return ${*$client}{'SSLeay'};
 }
 
@@ -280,6 +308,49 @@ sub read {
     return length $read;
 }
 
+sub sysread  {
+    my ($client, $buf, $length, $offset) = @_;
+    $length = length $buf unless defined $length;
+    $offset = 0 unless defined $offset;
+    my $ssl = $client->SSLeay;
+    my $data = Net::SSLeay::read($ssl, $length);
+
+    return if $!{EAGAIN} || $!{EINTR};
+
+    die "SSLeay print: $!\n" unless defined $data;
+
+    $length = length($data);
+    $$buf = '' if !defined $buf;
+
+    if ($offset > length($$buf)) {
+        $$buf .= "\0" x ($offset - length($buf));
+    }
+
+    substr($$buf, $offset, length($$buf), $data);
+    return $length;
+}
+
+sub error { my $client = shift; return ${*$client}{'_error'} }
+
+sub syswrite {
+    my ($client, $buf, $length, $offset) = @_;
+    delete ${*$client}{'_error'};
+
+    $length = length $buf unless defined $length;
+    $offset = 0 unless defined $offset;
+    my $ssl    = $client->SSLeay;
+
+    my $write = Net::SSLeay::write_partial($ssl, $offset, $length, $buf);
+
+    return if $!{EAGAIN} || $!{EINTR};
+    if ($write < 0) {
+        ${*$client}{'_error'} = "SSLeay print: $!\n";
+        return;
+    }
+
+    return $write;
+}
+
 sub getline {
     my $client = shift;
     my ($ok, $line) = $client->read_until($client->SSL_max_getline_length, $/);
@@ -299,6 +370,7 @@ sub getlines {
 
 sub print {
     my $client = shift;
+    delete ${*$client}{'_error'};
     my $buf    = @_ == 1 ? $_[0] : join('', @_);
     my $ssl    = $client->SSLeay;
     while (length $buf) {
@@ -307,7 +379,10 @@ sub print {
 
         my $write = Net::SSLeay::write($ssl, $buf);
         return 0 if $client->SSLeay_check_error('SSLeay write');
-        die "SSLeay print: $!\n" if $write == -1 && !$!{EAGAIN} && !$!{EINTR} && !$!{ENOBUFS};
+        if ($write == -1 && !$!{EAGAIN} && !$!{EINTR} && !$!{ENOBUFS}) {
+            ${*$client}{'_error'} = "SSLeay print: $!\n";
+            return;
+        }
         substr($buf, 0, $write, "") if $write > 0;
     }
     return 1;
@@ -330,20 +405,37 @@ sub write {
     $client->print($buf);
 }
 
-sub sysread  { die "sysread is not supported by Net::Server::Proto::SSLEAY" }
-sub syswrite { die "syswrite is not supported by Net::Server::Proto::SSLEAY" }
+sub seek {
+    my $client = shift;
+    my ($pos, $whence) = @_;
+    if ($whence) {
+        $! = "Seek from $whence of non-zero is not supported.";
+        return 0;
+    }
+    my $n = $client->read(my $buf, $pos);
+    if ($n != $pos) {
+        $| = "Couldn't seek to $pos ($n)\n";
+        return 0;
+    }
+    return 1;
+}
+
+sub poll_cb { # implemented for psgi compatibility - TODO - should poll appropriately for Multipex
+    my ($self, $cb) = @_;
+    return $cb->($self);
+}
 
 ###----------------------------------------------------------------###
 
 sub hup_string {
     my $sock = shift;
-    return join "|", map{$sock->$_()} qw(NS_host NS_port NS_proto);
+    return join "|", $sock->NS_host, $sock->NS_port, $sock->NS_proto, "ipv".$sock->NS_ipv, (defined(${*$sock}{'NS_orig_port'}) ? ${*$sock}{'NS_orig_port'} : ());
 }
 
 sub show {
     my $sock = shift;
-    my $t = "Ref = \"" .ref($sock) . "\"\n";
-    foreach my $prop ( qw(NS_proto NS_port NS_host SSLeay_context SSLeay_is_client) ){
+    my $t = "Ref = \"".ref($sock). "\" (".$sock->hup_string.")\n";
+    foreach my $prop (qw(SSLeay_context SSLeay_is_client)) {
         $t .= "  $prop = \"" .$sock->$prop()."\"\n";
     }
     return $t;
@@ -353,7 +445,7 @@ sub AUTOLOAD {
     my $sock = shift;
     my $prop = $AUTOLOAD =~ /::([^:]+)$/ ? $1 : die "Missing property in AUTOLOAD.";
     die "Unknown method or property [$prop]"
-        if $prop !~ /^(NS_proto|NS_port|NS_host|SSLeay_context|SSLeay_is_client|SSL_\w+)$/;
+        if $prop !~ /^(SSLeay_context|SSLeay_is_client|SSL_\w+)$/;
 
     no strict 'refs';
     *{__PACKAGE__."::${prop}"} = sub {
@@ -372,22 +464,71 @@ sub tie_stdout { 1 }
 
 1;
 
+__END__
+
 =head1 NAME
 
-Net::Server::Proto::SSLEAY - Custom Net::Server SSL protocol handler based on Net::SSLeay directly.
+Net::Server::Proto::SSLEAY - Custom Net::Server SSL protocol handler based on Net::SSLeay.
 
 =head1 SYNOPSIS
 
 See L<Net::Server::Proto>.
 
+    use base qw(Net::Server::HTTP);
+    main->run(
+        proto => 'ssleay',
+        SSL_key_file  => "/path/to/my/file.key",
+        SSL_cert_file => "/path/to/my/file.crt",
+    );
+
+
+    # OR
+
+    sub SSL_key_file  { "/path/to/my/file.key" }
+    sub SSL_cert_file { "/path/to/my/file.crt" }
+    main->run(proto => 'ssleay');
+
+
+    # OR
+
+    main->run(
+        port => [443, 8443, "80/tcp"],  # bind to two ssleay ports and one tcp
+        proto => "ssleay",    # use ssleay as the default
+        ipv  => "*",          # bind both IPv4 and IPv6 interfaces
+        SSL_key_file  => "/path/to/my/file.key",
+        SSL_cert_file => "/path/to/my/file.crt",
+    );
+
+
+    # OR
+
+    main->run(port => [{
+        port  => "443",
+        proto => "ssleay",
+        # ipv => 4, # default - only do IPv4
+        SSL_key_file  => "/path/to/my/file.key",
+        SSL_cert_file => "/path/to/my/file.crt",
+    }, {
+        port  => "8443",
+        proto => "ssleay",
+        ipv   => "*", # IPv4 and IPv6
+        SSL_key_file  => "/path/to/my/file2.key", # separate key
+        SSL_cert_file => "/path/to/my/file2.crt", # separate cert
+    }]);
+
+
 =head1 DESCRIPTION
 
-This module is relatively new and has only served a couple of months in production environments.
-If anybody has any successes or ideas for improvment under SSLEAY, please email <paul@seamons.com>.
+This module has reliably been used in situations receiving millions of
+hits on a single box per day.  If anybody has any successes or ideas
+for improvment under SSLEAY, please email <paul@seamons.com>.
 
-Protocol module for Net::Server.  This module implements a
-secure socket layer over tcp (also known as SSL).
-See L<Net::Server::Proto>.
+Protocol module for Net::Server.  This module implements a secure
+socket layer over tcp (also known as SSL).  See L<Net::Server::Proto>.
+
+If you need more customization of the SSL layer, you may want to
+investigate using SSL rather than SSLEAY as it uses the venerable(ish)
+IO::Socket::SSL.
 
 =head1 PARAMETERS
 
@@ -397,29 +538,31 @@ Currently there is support for the following:
 
 =item C<SSL_cert_file>
 
-Full path to the certificate file to be used for this server.  Should be in PEM format.
+Full path to the certificate file to be used for this server.  Should
+be in PEM format.
 
 =item C<SSL_key_file>
 
-Full path to the key file to be used for this server.  Should be in PEM format.
+Full path to the key file to be used for this server.  Should be in
+PEM format.
 
 =item C<SSL_max_getline_length>
 
-Used during getline to only read until this many bytes are found.  Default is undef which
-means unlimited.
+Used during getline to only read until this many bytes are found.
+Default is undef which means unlimited.
 
 =item C<SSL_error_callback>
 
-Should be a code ref that will be called whenever error conditions are encountered.  It passes a source message
-and an arrayref of the errors.
+Should be a code ref that will be called whenever error conditions are
+encountered.  It passes a source message and an arrayref of the
+errors.
 
 =back
 
-I'll add support for more as patches come in.
-
 =head1 METHODS
 
-This module implements most of the common file handle operations.  There are some additions though:
+This module implements most of the common file handle operations.
+There are some additions though:
 
 =over 4
 
@@ -430,6 +573,10 @@ that many bytes are found.  If match qr is defined, it will read until
 the buffer matches that qr.  If both are undefined, it will read until
 there is nothing left to read.
 
+=item C<error>
+
+If an error occurred while writing, this method will return that error.
+
 =back
 
 =head1 BUGS
@@ -444,6 +591,7 @@ Distributed under the same terms as Net::Server
 
 Thanks to Bilbo at
 http://devpit.org/wiki/OpenSSL_with_nonblocking_sockets_%28in_Perl%29
-for documenting a more reliable way of accepting and reading SSL connections.
+for documenting a more reliable way of accepting and reading SSL
+connections.
 
 =cut
